# -*- encoding : utf-8 -*-

# Created by Thomas Boerger on 2012-03-16.
# Copyright (c) 2012. All rights reserved.

require 'optparse'
require 'progressbar'

module Archiver
  class Cli
    attr_accessor :stdout
    attr_accessor :stderr
    attr_accessor :config
    attr_accessor :parser

    def initialize(argv, stdout, stderr)
      @stdout = stdout
      @stderr = stderr

      @config = {
        :verbose => false,
        :from => false,
        :into => false
      }

      @parser = OptionParser.new do |opt|
        opt.banner = "Usage: archiver [options]"

        opt.separator ""
        opt.separator "Mandatory options:"

        opt.on("-f", "--from DIR", "From directory name") do |dir|
          @config[:from] = dir
        end

        opt.on("-i", "--into DIR", "Into directory name") do |dir|
          @config[:into] = dir
        end

        opt.separator ""
        opt.separator "Optional options:"

        opt.on("-V", "--verbose", "Verbose output format") do
          @config[:verbose] = true
        end

        opt.on("-h", "--help", "Show this help message") do
          stdout.puts @parser
          exit 0
        end

        opt.on("-v", "--version", "Show current version") do
          stdout.puts "archiver v#{Version::STRING}"
          exit 0
        end
      end

      @parser.parse!

      unless @config[:from] or @config[:into]
        puts @parser
        exit 1
      end

      checks
      process
    end

    def checks
      unless File.directory? config[:from]
        error "Fromdir #{config[:from]} does not exist"
      end

      unless File.directory? config[:into]
        error "Intodir #{config[:into]} does not exist"
      end

      config[:fromdir] = File.realpath config[:from]
      config[:intodir] = File.realpath config[:into]
    end

    def process
      unless config[:verbose]
        progress = ProgressBar.new("Archiver", records.count, stdout)
      end

      records.each do |record|
        next unless record.process?

        if config[:verbose]
          from = record.path.gsub(/^#{config[:fromdir]}\//, "")
          into = File.join(record.segment, record.filename)

          stdout.puts "#{from} -> #{into}"
        else
          progress.inc
        end

        record.move config[:intodir]
      end

      if config[:verbose]
        stdout.puts "Processed #{records.count} records!"
      else
        progress.finish
      end
    rescue SystemExit, Interrupt
      progress.finish if progress
      exit 1
    end

    def records
      @records ||= begin
        rows = []
        Dir.glob(File.join(config[:fromdir], "**", "*")).each do |row|
          next if File.directory? row
          rows << Record.new(row)
        end

        rows
      end
    end

    def error(message)
      stderr.puts "Error: #{message}"
      stderr.puts @parser
      exit 1
    end
  end
end
